@@ -1,6 +1,6 @@
 package App::bmkpasswd;
 {
-  $App::bmkpasswd::VERSION = '1.081002';
+  $App::bmkpasswd::VERSION = '1.082000';
 }
 use strictures 1;
 use Carp;
@@ -12,7 +12,8 @@ use Crypt::Eksblowfish::Bcrypt qw/
 /;
 
 use Crypt::Random::Seed;
-my $crs = Crypt::Random::Seed->new;
+my $crs   = Crypt::Random::Seed->new;
+my $crsnb = Crypt::Random::Seed->new(NonBlocking => 1);
 
 use Exporter 'import';
 our @EXPORT_OK = qw/
@@ -58,27 +59,30 @@ sub have_sha {
     return $_can_haz{$type} = 1
   }
 
-  $_can_haz{$type} = 0
+  $_can_haz{$type} = 0;
+  return
 }
 
 
 sub _saltgen {
-  my ($type) = @_;
+  my ($type, $strong) = @_;
+
+  my $rnd = $strong ? $crs : $crsnb ;
 
   SALT: {
     if ($type eq 'bcrypt') {
-      return en_base64( $crs->random_bytes(16) );
+      return en_base64( $rnd->random_bytes(16) );
     }
 
     if ($type eq 'sha') {
-      my $max = en_base64( $crs->random_bytes(16) );
+      my $max = en_base64( $rnd->random_bytes(16) );
       my $initial = substr $max, 0, 8, '';
       $initial .= substr $max, 0, 1, '' for  1 .. rand 8;
       return $initial
     }
 
     if ($type eq 'md5') {
-      return en_base64( $crs->random_bytes(6) );
+      return en_base64( $rnd->random_bytes(6) );
     }
   }
 
@@ -86,7 +90,7 @@ sub _saltgen {
 }
 
 sub mkpasswd {
-  my ($pwd, $type, $cost) = @_;
+  my ($pwd, $type, $cost, $strong) = @_;
 
   $type = 'bcrypt' unless $type;
   my $salt;
@@ -99,7 +103,7 @@ sub mkpasswd {
         unless $cost =~ /^[0-9]+$/;
       $cost = '0$cost' if length $cost == 1;
 
-      $salt = _saltgen('bcrypt');
+      $salt = _saltgen('bcrypt', $strong);
       my $bsettings = join '', '$2a$', $cost, '$', $salt;
 
       return bcrypt($pwd, $bsettings)
@@ -109,19 +113,19 @@ sub mkpasswd {
     if ($type =~ /sha-?512/i) {
       croak "SHA hash requested but no SHA support available" 
         unless have_sha(512);
-      $salt = join '', '$6$', _saltgen('sha'), '$';
+      $salt = join '', '$6$', _saltgen('sha', $strong), '$';
       last TYPE
     }
 
     if ($type =~ /sha(-?256)?/i) {
       croak "SHA hash requested but no SHA support available" 
         unless have_sha(256);
-      $salt = join '', '$5$', _saltgen('sha'), '$';
+      $salt = join '', '$5$', _saltgen('sha', $strong), '$';
       last TYPE
     }
 
     if ($type =~ /^md5$/i) {
-      $salt = join '', '$1$', _saltgen('md5'), '$';
+      $salt = join '', '$1$', _saltgen('md5', $strong), '$';
       last TYPE
     }
 
@@ -198,8 +202,6 @@ are also exported for use in other applications; see L</EXPORTED>.)
 See C<bmkpasswd --help> for usage information.
 
 Uses L<Crypt::Random::Seed> to generate random salts.
-This means that systems with low entropy may block on B<mkpasswd> 
-(try L<http://www.issihosts.com/haveged/>).
 
 Uses L<Crypt::Eksblowfish::Bcrypt> for bcrypted passwords. Bcrypt hashes 
 come with a configurable work-cost factor; that allows hash generation 
@@ -238,6 +240,10 @@ other Perl modules/applications:
   $crypted = mkpasswd($passwd, 'sha256');
   $crypted = mkpasswd($passwd, 'sha512');
 
+  ## Use a strong random source (requires spare entropy):
+  $crypted = mkpasswd($passwd, 'bcrypt', '08', 1);
+  $crypted = mkpasswd($passwd, 'sha512', '', 1);
+
 =head2 passwdcmp
 
   ## Compare a password against a hash
